;-------------------------------------------------|
;           "Drawsprite v1.0"                     |
;Description: Subroutine for fast sprite-graphics |
;             for use with TI-82, with ASH 3.0    |
;-------------------------------------------------|
;-----------------------------------------|
;Author: Jakob Selbing, JSS (C) 1997-1998 |
;  Comments, bugs, tips, anything on      |
;  your mind:                             |
;  jakob_selbing@hotmail.com              |
;-----------------------------------------|
;
;----------------------------------------------------|
;Credits: ASH-guys Dines and Mattias, Vikash R. Goel |
;----------------------------------------------------|                                                   
;
;How to use it 
;----------------------------------------------------------------
;Call drawspriteloop with the x-position (0...88) stored in a,
;the offset ($88B8...$88B8+767) in the graphics memory stored in hl,
;the address of the sprite stored in bc, and the number of rows your
;sprite is stored in temp.
;
;So if you want to draw a sprite on the third row, 20 pixels to the rigth,
;and have defined a sprite with 6 rows at the label "sprite1" you
;should do something like this:
;
; ld a,6
; ld (temp),a
; ld a,20
; ld hl,$88DC
; ld bc,sprite1
; call drawspriteloop
; ROM_CALL(DISP_GRAPH)
;
;Remember that you must define "temp" as an address to the RAM
;where nothing else is stored. I use TEXT_MEM.
;
;Please include a line with my name and e-mail in your program!!!
;I have lost a few hours of sleep over this...
;-----------------------------------------------------------------------

#include "ti82.h"
temp    =TEXT_MEM ;not necessarily TEXT_MEM

;"Drawsprite" written by Jakob Selbing, jakob_selbing@hotmail.com

drawspriteloop:  ;a=x-position          hl=offset in graph mem
 ld de,0         ;bc=sprite's address   temp=number of rows
getcolumn:
 cp 8            ;Check if a=8 or more
 jr c,getbit     ;If not, go to getbit
 sub 8           ;Subtract 8 from a
 inc e           ;Increase the number of bytes to add to GRAPH_MEM
 jr getcolumn    ;Do this until a<8
getbit:          
 add hl,de       ;Add the number of whole bytes
putnewrow:       
 push af         ;Save the number of shifts to do
 push bc         ;Save the address of the sprite
 push hl         ;Save address of place in GRAPH_MEM
 ld h,b          ;Get the address to sprite
 ld l,c          ;Get the address to sprite
 ld b,(hl)       ;Load row no. (temp) of sprite in b
 ld c,0          ;We don't want any random pixels on the screen!
rotate:         
 or a            ;Check if all the shiftings are done
 jr z,donerotate ;If so, go to donerotate
 dec a           ;Else decrease the number of shifts left to do
 srl b           ;Shift bc rigth
 rr c            ;Shift bc rigth
 jr rotate       ;Do another one (if a>0)
donerotate:
 pop hl          ;Get the address in GRAPH_MEM where we'll load sprite 
 ld (hl),b       ;Load first 8 pixels (bits) of sprite in mem
 inc hl          ;Move to next byte in GRAPH_MEM
 ld (hl),c       ;Load last 8 pixels of sprite in mem
 pop bc          ;Get the address of sprite back
 inc bc          ;Point to next byte (row) of sprite
 ld de,11        ;Move to next row of display
 add hl,de       ;Move to next row of display
 ld a,(temp)     ;Load the number of rows left
 dec a           ;Decrease it since we just drew a row
 ld (temp),a     ;Save it
 or a            ;Are we done?
 jr z,endofsprite;If so jump to endofsprite
 pop af          ;Else, get that x-position again
 jr putnewrow    ;...and do it all over again
endofsprite:
 pop af          ;This needs to be done, otherwise it gets really nasty
 ret             ;Return from the subroutine
